==> mdcached-1.0.2.tgz: mdcached-1.0.2/mdcached/ServerClientConnection.h <==
/*-
 * This file is a part of the MDCacheD server
 * (c) 2007.-2011. Ivan Voras <ivoras@gmail.com>
 * Released under the 2-clause BSDL
 * $Id: ServerClientConnection.h 347 2011-09-20 20:58:41Z ivoras $
 */

#ifndef _SERVER_CLIENT_CONNECTION_H_
#define _SERVER_CLIENT_CONNECTION_H_

#include "minilibevent.h"
#include <sys/time.h>


/* A lot of dirty connections to server.cpp since all of this was refactored
 * from it. */
extern time_t current_time;
#ifndef NDEBUG
extern struct debug_status_record global_st;
#endif


enum E_CONN_TYPE {
	CONN_TYPE_NO = 0,
	CONN_TYPE_CLIENT_UNIX,
	CONN_TYPE_CLIENT_TCP,
	CONN_TYPE_CLIENT_UDP, /* a fake connection */
	CONN_TYPE_SERVER_UNIX,
	CONN_TYPE_SERVER_TCP,
	CONN_TYPE_SERVER_UDP
};

const char *conn_type_name(enum E_CONN_TYPE type);

/** Binary OR flags to the struct md_conn */
enum E_CONN_FLAG { ZERO, CONN_DONE = 1, CONN_FAKE_DATAGRAM = 2 };

enum EVENT_PROC_RESULT { EVPROC_OK, EVPROC_ERROR };

/* INBUF and OUTBUF sizes should be big enough to handle *most* data, to avoid
 * excessive mallocs and reallocs. */
#ifndef INBUF_ALLOC_SIZE
#define INBUF_ALLOC_SIZE 1500
#endif

#ifndef OUTBUF_ALLOC_SIZE
#define OUTBUF_ALLOC_SIZE INBUF_ALLOC_SIZE
#endif

#ifndef SNDBUF_SIZE
/**
 * SNDBUF_SIZE and RCVBUF_SIZE should be large enough to hold
 * almost all request and response sizes. Outgoing packets that
 * are larger than SNDBUF_SIZE will be sent in pieces by the
 * network thread instead of worker threads.
 */
#define SNDBUF_SIZE (4 * OUTBUF_ALLOC_SIZE)
#endif

#ifndef RCVBUF_SIZE
#define RCVBUF_SIZE (4 * INBUF_ALLOC_SIZE)
#endif


/**
 * Connection descriptors - every established connection (as well as the server
 * "listening" connections) is represented by one of these. It is also the
 * focus of connection IO with the buffers and buffer position indexes.
 */
class ServerClientConnection {
public:
#ifndef NDEBUG
	struct debug_status_record st;
#endif
	enum E_CONN_TYPE 		type;	/* type of this connection descriptor */
	enum E_CONN_FLAG		flags;
	int				socket;
	int				index;
	union {
		struct sockaddr_in	sin;
		struct sockaddr_un	sun;
	};

	ServerClientConnection() {
		_init_defaults();
	}

	ServerClientConnection(int index) {
		_init_defaults();
		this->index = index;
	}

	ServerClientConnection(int socket, enum E_CONN_TYPE type, int index) {
		_init_defaults();
		this->socket = socket;
		this->type = type;
		this->index = index;
	}

	~ServerClientConnection() {
		if (inbuf != NULL)
			free(inbuf);
		if (outbuf != NULL)
			free(outbuf);
		inbuf = NULL;
		outbuf = NULL;
		if (socket != 0)
			close(socket);
		index = socket = 0;
		type = CONN_TYPE_NO;
	}

	void assure_bufs(bool init_outbuf);
	void new_inbuf();
	void setup_blank(int index);
	void disconnect_client();
	int output(void *buf, unsigned int buf_size, bool clear_inbuf);
	void send_status(int seq, int status, int data);
	void process_command();
	enum EVENT_PROC_RESULT process_read_event(ServerNetworkThread *nthr);
	enum EVENT_PROC_RESULT process_write_event();
	int enable_read_event();
	int disable_read_event();
	int enable_write_event();
	int disable_write_event();

	unsigned int outputDataReady() {
		return outbuf_len - outbuf_pos;
	}
	bool inbufValid() {
		return inbuf != NULL;
	}
	void inbufReset() {
		inbuf_pos = 0;
	}
	void setEvq(ev_queue *qp) {
		myevq = qp;
	}

private:
	uint32_t		tsd_counter;
	char			*outbuf;
	unsigned int		 outbuf_len;	/* total buffer size */
	unsigned int		 outbuf_pos;	/* pointer to next data */
	struct mc_header	*inbuf;
	unsigned int		 inbuf_len;	/* total buffer size */
	unsigned int		 inbuf_pos;	/* used (data) size */
	ev_queue		*myevq;

	void queue_work(ServerNetworkThread *nthr);
	void offload_command_packet(ServerNetworkThread *nthr);
	int cmd_handshake();
	int cmd_put_data();
	int cmd_get_data_name();
	int cmd_del_data_name();
	int cmd_get_data_by_tag_values();
	int cmd_del_data_by_tag_values();
	int cmd_get_all_keys();
	int cmd_error_inv_command();
	int cmd_atomic();
	int cmd_mput();
	int cmd_mget();
	int cmd_mdel();
	int cmd_tstack_push();
	int cmd_tstack_pop();

	void _init_defaults() {
		inbuf = NULL;
		outbuf = NULL;
		outbuf_pos = inbuf_pos = outbuf_len = inbuf_len = 0;
		socket = index = 0;
		myevq = NULL;
		tsd_counter = 0;
	}
};

#endif

==> mdcached-1.0.3.tgz: mdcached-1.0.3/mdcached/ServerClientConnection.h <==
/*-
 * This file is a part of the MDCacheD server
 * (c) 2007.-2011. Ivan Voras <ivoras@gmail.com>
 * Released under the 2-clause BSDL
 * $Id: ServerClientConnection.h 430 2012-03-01 13:09:48Z ivoras $
 */

#ifndef _SERVER_CLIENT_CONNECTION_H_
#define _SERVER_CLIENT_CONNECTION_H_

#include "minilibevent.h"
#include <sys/time.h>


/* A lot of dirty connections to server.cpp since all of this was refactored
 * from it. */
extern time_t current_time;
#ifndef NDEBUG
extern struct debug_status_record global_st;
#endif


enum E_CONN_TYPE {
	CONN_TYPE_NO = 0,
	CONN_TYPE_CLIENT_UNIX,
	CONN_TYPE_CLIENT_TCP,
	CONN_TYPE_CLIENT_UDP, /* a fake connection */
	CONN_TYPE_SERVER_UNIX,
	CONN_TYPE_SERVER_TCP,
	CONN_TYPE_SERVER_UDP
};

const char *conn_type_name(enum E_CONN_TYPE type);

/** Binary OR flags to the struct md_conn */
enum E_CONN_FLAG { ZERO, CONN_DONE = 1, CONN_FAKE_DATAGRAM = 2 };

enum EVENT_PROC_RESULT { EVPROC_OK, EVPROC_ERROR };

/* INBUF and OUTBUF sizes should be big enough to handle *most* data, to avoid
 * excessive mallocs and reallocs. */
#ifndef DEFAULT_INBUF_ALLOC_SIZE
#define DEFAULT_INBUF_ALLOC_SIZE 1500
#endif

#ifndef DEFAULT_OUTBUF_ALLOC_SIZE
#define DEFAULT_OUTBUF_ALLOC_SIZE DEFAULT_INBUF_ALLOC_SIZE
#endif

#ifndef SNDBUF_SIZE
/**
 * SNDBUF_SIZE and RCVBUF_SIZE should be large enough to hold
 * almost all request and response sizes. Outgoing packets that
 * are larger than SNDBUF_SIZE will be sent in pieces by the
 * network thread instead of worker threads.
 */
#define SNDBUF_SIZE (4 * DEFAULT_OUTBUF_ALLOC_SIZE)
#endif

#ifndef RCVBUF_SIZE
#define RCVBUF_SIZE (4 * DEFAULT_INBUF_ALLOC_SIZE)
#endif


/**
 * Connection descriptors - every established connection (as well as the server
 * "listening" connections) is represented by one of these. It is also the
 * focus of connection IO with the buffers and buffer position indexes.
 */
class ServerClientConnection {
public:
#ifndef NDEBUG
	struct debug_status_record st;
#endif
	enum E_CONN_TYPE 		type;	/* type of this connection descriptor */
	enum E_CONN_FLAG		flags;
	int				socket;
	int				index;
	union {
		struct sockaddr_in	sin;
		struct sockaddr_un	sun;
	};

	ServerClientConnection() {
		_init_defaults();
	}

	ServerClientConnection(int index) {
		_init_defaults();
		this->index = index;
	}

	ServerClientConnection(int socket, enum E_CONN_TYPE type, int index) {
		_init_defaults();
		this->socket = socket;
		this->type = type;
		this->index = index;
	}

	~ServerClientConnection() {
		if (inbuf != NULL)
			free(inbuf);
		if (outbuf != NULL)
			free(outbuf);
		inbuf = NULL;
		outbuf = NULL;
		if (socket != 0)
			close(socket);
		index = socket = 0;
		type = CONN_TYPE_NO;
	}

	void assure_bufs(bool init_outbuf);
	void new_inbuf();
	void setup_blank(int index);
	void disconnect_client();
	int output(void *buf, unsigned int buf_size, bool clear_inbuf);
	void send_status(int seq, int status, int data);
	void process_command();
	enum EVENT_PROC_RESULT process_read_event(ServerNetworkThread *nthr);
	enum EVENT_PROC_RESULT process_write_event();
	int enable_read_event();
	int disable_read_event();
	int enable_write_event();
	int disable_write_event();

	unsigned int outputDataReady() {
		return outbuf_len - outbuf_pos;
	}
	bool inbufValid() {
		return inbuf != NULL;
	}
	void inbufReset() {
		inbuf_pos = 0;
	}
	void setEvq(ev_queue *qp) {
		myevq = qp;
	}

private:
	uint32_t		tsd_counter;
	char			*outbuf;
	unsigned int		 outbuf_len;	/* total buffer size */
	unsigned int		 outbuf_pos;	/* pointer to next data */
	struct mc_header	*inbuf;
	unsigned int		 inbuf_len;	/* total buffer size */
	unsigned int		 inbuf_pos;	/* used (data) size */
	ev_queue		*myevq;

	void queue_work(ServerNetworkThread *nthr);
	void offload_command_packet(ServerNetworkThread *nthr);
	int cmd_handshake();
	int cmd_put_data();
	int cmd_get_data_name();
	int cmd_del_data_name();
	int cmd_get_data_by_tag_values();
	int cmd_del_data_by_tag_values();
	int cmd_get_all_keys();
	int cmd_error_inv_command();
	int cmd_atomic();
	int cmd_mput();
	int cmd_mget();
	int cmd_mdel();
	int cmd_tstack_push();
	int cmd_tstack_pop();

	void _init_defaults() {
		inbuf = NULL;
		outbuf = NULL;
		outbuf_pos = inbuf_pos = outbuf_len = inbuf_len = 0;
		socket = index = 0;
		myevq = NULL;
		tsd_counter = 0;
	}
};

#endif
